package com.sillycycle.bagleyd.abacus.learn;

/*
 * @(#)AbacusDemo.java
 *
 * Copyright 1994 - 2017  David A. Bagley, bagleyd AT verizon.net
 *
 * Abacus demo and neat pointers from
 * Copyright 1991 - 1998  Luis Fernandes, elf AT ee.ryerson.ca
 *
 * All rights reserved.
 *
 * Permission to use, copy, modify, and distribute this software and
 * its documentation for any purpose and without fee is hereby granted,
 * provided that the above copyright notice appear in all copies and
 * that both that copyright notice and this permission notice appear in
 * supporting documentation, and that the name of the author not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.
 *
 * This program is distributed in the hope that it will be "useful",
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 */

import javax.swing.JApplet;

import com.sillycycle.bagleyd.abacus.AbacusApplet;
import com.sillycycle.bagleyd.abacus.AbacusCanvas;
import com.sillycycle.bagleyd.abacus.AbacusInterface;

/**
 * The <code>AbacusDemo</code> class is shows the demonstration of
 * the <code>AbacusApplet</code> class to make it easier to learn
 * how to manipulate an abacus.
 *
 * @author	David A. Bagley
 * @author	bagleyd AT verizon.net
 * @author	http://www.sillycycle.com/abacus.html
 */

/* sed "s/^/             \"/" Abacus.les | sed "s/$/\",/" */

public class AbacusDemo {
	JApplet applet;
	AbacusCanvas abacus;
	static final boolean DEBUG = false;

	boolean bookTitle = true, chapterTitle = true;
	boolean query = false, started = false;
	int mode = 0, lessonCount = 0, lessonLine = 0;
	int bookCount = 0, chapterCount = 0;
	int highlightAux = -1, highlightRail = -1;

	public static final String NL = System.getProperty("line.separator");
	static final String LONG_STRING = "                                                       ";
	static final int MAX_LINES = 4;
	static final int INTRO = 0;
	static final int BOOK = 1;
	static final int CHAPTER = 2;
	static final int QUERY = 3;
	static final int DISPLAY = 4;
	static final int CONCL = 5;

	String libraryText[][][][][][] = null;
	String booksFromLibrary[] = null;
	String editionsFromLibrary[][] = null;
	String chaptersFromLibrary[][][] = null;

	String introFramedText[] = {
		"Click & leave the pointer in the abacus window" + LONG_STRING,
		"to begin the demo.",
		"During the demo, use the Space-bar to step.",
		"Type `o' to toggle demo or `q' to quit."
	};

	String bookText[][] = null;
	String chapterText[][][][] = null;

	String queryText[] = {
		"Type: `n' for the Next lesson" + LONG_STRING,
		"      `r' to Repeat previous lesson",
		"      `j' to Jump to next chapter",
		"      `q' to Quit the demo",
	};

	String conclLesson[] = {
		"Here Endth the Lesson",
		"",
		"",
		""
	};
	String displayText[] = new String[MAX_LINES];
	AbacusFallback abacusFallback = null;

	public AbacusDemo(AbacusApplet applet, AbacusCanvas abacus) {
		this.applet = applet;
		this.abacus = abacus;
		applet.readFile();
		AbacusExamples.setLibrary(AbacusApplet.getDoc());
		libraryText = AbacusExamples.getLibrary();
		booksFromLibrary = AbacusExamples.getBooks();
		editionsFromLibrary = AbacusExamples.getEditions();
		chaptersFromLibrary = AbacusExamples.getChapters();
		if (booksFromLibrary != null) {
			makeBookText(booksFromLibrary);
			if (chaptersFromLibrary != null) {
				makeChapterText(chaptersFromLibrary);
			}
		}
		if (booksFromLibrary == null || chaptersFromLibrary == null) {
			System.out.println("Using fallback");
			if (abacusFallback == null)
				abacusFallback = new AbacusFallback();
			bookText = abacusFallback.bookText;
			chapterText = abacusFallback.chapterText;
		}
	}

	static private int commonEdition(int mode) {
		if (mode == AbacusInterface.Modes.Danish.ordinal())
			return AbacusInterface.Modes.Russian.ordinal();
		else if (mode == AbacusInterface.Modes.Roman.ordinal())
			return AbacusInterface.Modes.Japanese.ordinal();
		else if (mode == AbacusInterface.Modes.Medieval.ordinal())
			return AbacusInterface.Modes.Chinese.ordinal();
		return mode;
	}

	private int getNumberBooks() {
		return booksFromLibrary.length;
	}

	private int getNumberEditions(int book) {
		return chaptersFromLibrary[book].length;
	}

	private int getEdition(int book, int abacusMode) {
        	int edition = commonEdition(abacusMode);
        	int editions = getNumberEditions(book);

        	if (edition >= editions) {
			edition = AbacusInterface.Modes.Chinese.ordinal();
		}
        	return edition;
	}

	private int getNumberChapters(int book, int abacusMode) {
		return chaptersFromLibrary[book][abacusMode].length;
	}

	private void makeBookText(String[] text) {
		bookText = new String[text.length][];
		for (int book = 0; book < text.length; book++) {
			bookText[book] = new String[MAX_LINES];
			bookText[book][0] = "     " + text[book] + LONG_STRING;
			bookText[book][1] = "Type: `r' to Read this book";
			bookText[book][2] = "      `n' to Next book";
			bookText[book][3] = "      `q' to Quit the demo";
		}
	}

	private void makeChapterText(String[][][] text) {
		chapterText = new String[text.length][][][];
		for (int book = 0; book < getNumberBooks(); book++) {
			chapterText[book] = new String[text[book].length][][];
			for (int edition = 0; edition < getNumberEditions(book); edition++) {
				chapterText[book][edition] = new String[text[book][edition].length][];
				for (int chapter = 0; chapter < text[book][edition].length; chapter++) {
					chapterText[book][edition][chapter] = new String[MAX_LINES];
					chapterText[book][edition][chapter][0] = "     " + text[book][edition][chapter] + LONG_STRING;
					chapterText[book][edition][chapter][1] = "Type: `r' to Read this chapter";
					chapterText[book][edition][chapter][2] = "      `n' to Next chapter";
					chapterText[book][edition][chapter][3] = "      `q' to Quit the demo";
				}
			}
		}
	}

	public boolean queryBook() {
		return bookTitle;
	}

	public boolean queryChapter() {
		return chapterTitle;
	}

	public String formatText(int textType) {
		int line;
		String[] tempText = new String[MAX_LINES];
		if (textType == CHAPTER) {
			int realMode = abacus.getMode();
			int forcedMode = getEdition(bookCount, realMode);
			mode = forcedMode;
			/* Korean and Japanese are similar and may be OK */
			if (forcedMode == 0 &&
					(realMode == AbacusInterface.Modes.Russian.ordinal() ||
					realMode == AbacusInterface.Modes.Danish.ordinal())) {
				System.out.println("No Russian edition, expect errors.");
			}
		}
		for (line = 0; line < MAX_LINES; line++) {
			switch (textType) {
			case INTRO:
				tempText[line] = introFramedText[line];
				if (DEBUG)
					System.out.println(" INTRO: " +
						tempText[line]);
				chapterTitle = true;
				break;
			case BOOK:
				tempText[line] =
					bookText[bookCount][line];
				if (DEBUG)
					System.out.println(" BOOK: " +
						tempText[line]);
				break;
			case CHAPTER:
				tempText[line] =
					chapterText[bookCount][mode][chapterCount][line];
				if (DEBUG)
					System.out.println(" CHAPTER: " +
						tempText[line]);
				break;
			case CONCL:
				started = false;
				tempText[line] = conclLesson[line];
				if (DEBUG)
					System.out.println(" CONCL: " +
						tempText[line]);
				break;
			case QUERY:
				query = true;
				tempText[line] = queryText[line];
				if (DEBUG)
					System.out.println(" QUERY: " +
						tempText[line]);
				break;
			default:
				tempText[line] = displayText[line];
				if (DEBUG)
					System.out.println(" DISPLAY: " +
						tempText[line]);
				break;
			}
		}
		return tempText[0] + NL + tempText[1] + NL +
			tempText[2] + NL + tempText[3];
	}

	public void drawText(int textType) {
		((AbacusApplet)applet).showDemoMsg(formatText(textType));
	}

	public void drawText(int textType0, int textType1, int delay) {
		((AbacusApplet)applet).showDemoMsg2(formatText(textType0),
			formatText(textType1), delay);
	}

	public void startLesson() {
		lessonLine = 0;
		mode = getEdition(bookCount, abacus.getMode());
	}

	public boolean doLessonLines() {
		if (libraryText[bookCount][mode][chapterCount].length <= 0) {
			chapterTitle = true;
			/*System.err.println("AbacusDemo: chapterCount " +
				libraryText[bookCount][mode][chapterCount].length);*/
			return false;
		}
		String[] strings =
			libraryText[bookCount][mode][chapterCount][lessonCount][lessonLine][0].split(" ");
		int aux, deck, rail, number, lines;

		if (5 <= strings.length) {
			aux = Integer.parseInt(strings[0]);
			deck = Integer.parseInt(strings[1]);
			rail = Integer.parseInt(strings[2]);
			number = Integer.parseInt(strings[3]);
			lines = Integer.parseInt(strings[4]);
		} else {
			System.err.println("AbacusDemo: size " + strings.length);
			return false;
		}
		if (DEBUG)
			System.out.println("aux = " + aux +
				", deck = " + deck +
				", rail = " + rail +
				", number = " + number +
				", lines = " + lines);
		abacus.abacusMove(aux, deck, rail, number);
		for (int line = 0; line < MAX_LINES; line++) {
			if (line >= lines) {
				displayText[line] = "";
			} else {
				displayText[line] =
					libraryText[bookCount][mode][chapterCount][lessonCount][lessonLine][line + 1];
			}
		}
		if (highlightAux >= 0 && highlightRail >= 0) {
			abacus.highlightRail(highlightAux, highlightRail, false);
			highlightAux = -1;
			highlightRail = -1;
		}
		if (aux >= 0 && rail >= 0) {
			if (number == 0) {
				highlightAux = -1;
				highlightRail = -1;
			} else {
				highlightAux = aux;
				highlightRail = rail;
				abacus.highlightRail(aux, rail, true);
			}
		}
		if (DEBUG)
			System.out.println(" doLessonLines");
		return true;
	}
 
	public void doLesson() {
		if (libraryText == null) {
			libraryText = new String[1][][][][][];
			if (abacusFallback == null)
				abacusFallback = new AbacusFallback();
			bookCount = 0;
			libraryText[bookCount] = abacusFallback.lessonText;
			System.err.println("Using fallback demo");
		}
		if (libraryText[bookCount][mode][chapterCount].length > 0 &&
				lessonLine == libraryText[bookCount][mode][chapterCount][lessonCount].length) {
			drawText(CONCL, QUERY, 2000);
			if (highlightAux >= 0 && highlightRail >= 0) {
				abacus.highlightRail(highlightAux, highlightRail, false);
				highlightAux = -1;
				highlightRail = -1;
        		}
			lessonLine = 0;
			abacus.clearRails();
		} else if (started) {
			if (bookTitle) {
				drawText(BOOK);
			} else if (chapterTitle) {
				drawText(CHAPTER);
			} else {
				if (doLessonLines()) {
					lessonLine++;
					drawText(DISPLAY);
				}
			}
		} else if (query) {
			drawText(QUERY);
		} else {
			drawText(INTRO);
		}
	}

	public void clearDemo() {
		bookCount = chapterCount = 0;
		bookTitle = chapterTitle = true;
		started = true;
		query = false;
		if (highlightAux >= 0 && highlightRail >= 0) {
			abacus.highlightRail(highlightAux, highlightRail, false);
			highlightAux = -1;
			highlightRail = -1;
        	}
		startLesson();
		if (chaptersFromLibrary != null && chaptersFromLibrary[0] != null)
			doLesson();
	}

	public void queryDemo(boolean advance) {
		if (DEBUG)
			System.out.println("queryDemo: advance " + advance);
		started = true;
		if (queryBook()) {
			if (advance) {
				bookCount++;
				if (bookCount >= getNumberBooks()) {
					bookCount = 0;
				}
				drawText(BOOK);
			} else {
				/* got a Read */
				bookTitle = false;
				chapterTitle = true;
				chapterCount = 0;
				query = false;
				drawText(CHAPTER);
			}
		} else if (queryChapter()) {
			if (advance) {
				chapterCount++;
				if (chapterCount >= getNumberChapters(bookCount, mode)) {
					chapterCount = 0;
					bookCount++;
					if (bookCount >= getNumberBooks()) {
						bookCount = 0;
					}
					bookTitle = true;
					drawText(BOOK);
				} else {
					drawText(CHAPTER);
				}
			} else {
				/* got a Read */
				chapterTitle = false;
				lessonCount = 0;
				query = false;
				startLesson();
				doLesson();
			}
		} else {
			if (query) {
				started = true;
				query = false;
				if (advance) {
					lessonCount++;
					if (lessonCount >= libraryText[bookCount][mode][chapterCount].length) {
						lessonCount = 0;
						chapterCount++;
						if (chapterCount >= getNumberChapters(bookCount, mode)) {
							chapterCount = 0;
							bookCount++;
							if (bookCount >= getNumberBooks()) {
								bookCount = 0;
							}
							bookTitle = true;
							drawText(BOOK);
						} else {
							chapterTitle = true;
							drawText(CHAPTER);
						}
					}
				}
				startLesson();
				doLesson();
			}
		}
	}

	public void jumpDemo() {
		if (!chapterTitle) {
			chapterTitle = true;
			started = true;
			chapterCount++;
			if (chapterCount >= getNumberChapters(bookCount, mode)) {
				chapterCount = 0;
				bookCount++;
				if (bookCount >= getNumberBooks()) {
					bookCount = 0;
				}
				bookTitle = true;
				drawText(BOOK);
			} else {
				drawText(CHAPTER);
			}
		}
	}

	public void chapterDemo(int chapt) {
		if (started && chapterTitle) {
			chapterCount = chapt;
			if (chapterCount < 0 ||
					chapterCount >= getNumberChapters(bookCount, mode))
				chapterCount = 0;
			drawText(CHAPTER);
		}
	}

	public void moreDemo() {
		if (!chapterTitle && started && !query) {
			doLesson();
		}
	}
}
