package com.sillycycle.bagleyd.abacus.learn;

/*
 * @(#)AbacusExamples.java
 *
 * Copyright 2015 - 2017  David A. Bagley, bagleyd AT verizon.net
 *
 * All rights reserved.
 *
 * Permission to use, copy, modify, and distribute this software and
 * its documentation for any purpose and without fee is hereby granted,
 * provided that the above copyright notice appear in all copies and
 * that both that copyright notice and this permission notice appear in
 * supporting documentation, and that the name of the author not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.
 *
 * This program is distributed in the hope that it will be "useful",
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 */

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.w3c.dom.Node;

/**
 * The <code>AbacusExamples</code> class handles the examples for Abacus.
 *
 * @author	David A. Bagley
 * @author	bagleyd AT verizon.net
 * @author	http://www.sillycycle.com/life.html
 */

public class AbacusExamples {
	static final boolean DEBUG = false;

	private static String libraryText[][][][][][] = null;
	private static String libraryBooks[] = null;
	private static String libraryEditions[][] = null;
	private static String libraryChapters[][][] = null;

	public static void setLibrary(Document doc) {
		libraryText = readExamplesXML(doc);
	}

	static String[][][][][][] readExamplesXML(Document doc) {
		if (doc == null)
			return null;
		// normalize text representation
		doc.getDocumentElement().normalize();
		/*if (DEBUG)
			System.out.println("Root element of the doc is " +
				 doc.getDocumentElement().getNodeName());*/
		NodeList listOfBooks = doc.getElementsByTagName("book");
		int totalBooks = listOfBooks.getLength();

		if (DEBUG)
			System.out.println("\n\nNumber of books: " + totalBooks);

		if (libraryBooks == null) {
			libraryBooks = new String[totalBooks];
		}
		if (libraryEditions == null) {
			libraryEditions = new String[totalBooks][];
		}
		if (libraryChapters == null) {
			libraryChapters = new String[totalBooks][][];
		}
		String[][][][][][] booksText = new String[totalBooks][][][][][];
		if (libraryBooks == null) {
			libraryBooks = new String[totalBooks];
		} else if (libraryBooks.length != totalBooks) {
			System.out.println("Old number of books "
				+ libraryBooks.length
				+ " does not match " + totalBooks);
		}
		for (int i = 0; i < totalBooks; i++) {
			Node bookNode = listOfBooks.item(i);
			if (bookNode.getNodeType() == Node.ELEMENT_NODE) {
				booksText[i] = readBook(bookNode, i);
			}
		}
		return booksText;
	}

	static String[][][][][] readBook(Node bookNode, int book) {
		try {
			Element bookElement = (Element) bookNode;
			String name, title, author;
			Node namedItem = bookElement.getAttributes().getNamedItem("name");
			title = (namedItem == null) ? "" : namedItem.getNodeValue();
			Node authorItem = bookElement.getAttributes().getNamedItem("author");
			author = (authorItem == null) ? "" : authorItem.getNodeValue();
			name = title + " by " + author;
			if (libraryBooks[book] == null) {
				libraryBooks[book] = name;
			} else if (!libraryBooks[book].equals(name)) {
				System.out.println("Old book name "
					+ libraryBooks[book]
					+ " does not match " + name);
			}
			NodeList listOfEditions = bookElement.getElementsByTagName("edition");
			int totalEditions = listOfEditions.getLength();
			if (libraryEditions[book] == null) {
				libraryEditions[book] = new String[totalEditions];
			} else if (libraryEditions[book].length != totalEditions) {
				System.out.println("Old number of editions "
					+ libraryEditions[book].length
					+ " does not match " + totalEditions);
			}
			if (libraryChapters[book] == null) {
				libraryChapters[book] = new String[totalEditions][];
			} else if (libraryChapters[book].length != totalEditions) {
				System.out.println("Old number of editions "
					+ libraryChapters[book].length
					+ " does not match " + totalEditions);
			}

			if (DEBUG)
				System.out.println("\nBook name: " + name
					+ ", author: " + author
					+ ", number of editions: " + totalEditions);

			String[][][][][] editionsText = new String[totalEditions][][][][];
			for (int i = 0; i < totalEditions; i++) {
				Node editionNode = listOfEditions.item(i);
				if (editionNode.getNodeType() == Node.ELEMENT_NODE) {
					editionsText[i] = readEdition(editionNode, book, i);
				}
			}
			return editionsText;
		} catch (Exception e) {
			e.printStackTrace();
		}
		return null;
	}

	static String[][][][] readEdition(Node editionNode,
			int book, int edition) {
		try {
			Element editionElement = (Element) editionNode;
			String version;
			Node versionItem = editionElement.getAttributes().getNamedItem("version");
			version = (versionItem == null) ? "" : versionItem.getNodeValue();
			if (libraryEditions[book][edition] == null) {
				libraryEditions[book][edition] = version;
			} else if (!libraryEditions[book][edition].equals(version)) {
				System.out.println("Old edition name "
					+ libraryEditions[book][edition]
					+ " does not match " + version);
			}
			NodeList listOfChapters = editionElement.getElementsByTagName("chapter");
			int totalChapters = listOfChapters.getLength();
			if (libraryChapters[book][edition] == null) {
				libraryChapters[book][edition] = new String[totalChapters];
			} else if (libraryChapters[book][edition].length != totalChapters) {
				System.out.println("Old number of chapters "
					+ libraryChapters[book][edition].length
					+ " does not match " + totalChapters);
			}

			if (DEBUG)
				System.out.println("\tEdition version: " + version
					+ ", number of chapters: " + totalChapters);

			String[][][][] chaptersText = new String[totalChapters][][][];
			for (int i = 0; i < totalChapters; i++) {
				Node chapterNode = listOfChapters.item(i);
				if (chapterNode.getNodeType() == Node.ELEMENT_NODE) {
					chaptersText[i] = readChapter(chapterNode, book, edition, i);
				}
			}
			return chaptersText;
		} catch (Exception e) {
			e.printStackTrace();
		}
		return null;
	}

	static String[][][] readChapter(Node chapterNode,
			int book, int edition, int chapter) {
		try {
			Element chapterElement = (Element) chapterNode;
			String name;
			Node namedItem = chapterElement.getAttributes().getNamedItem("name");
			name = (namedItem == null) ? "" : namedItem.getNodeValue();
			if (libraryChapters[book][edition][chapter] == null) {
				libraryChapters[book][edition][chapter] = name;
			} else if (!libraryChapters[book][edition][chapter].equals(name)) {
				System.out.println("Old chapter name "
					+ libraryChapters[book][chapter]
					+ " does not match " + name);
			}
			NodeList listOfLessons = chapterElement.getElementsByTagName("lesson");
			int totalLessons = listOfLessons.getLength();

			if (DEBUG)
				System.out.println("\t\tChapter name: " + name
					+ ", number of lessons: " + totalLessons);

			String[][][] lessonsText = new String[totalLessons][][];
			for (int i = 0; i < totalLessons; i++) {
				Node lessonNode = listOfLessons.item(i);
				if (lessonNode.getNodeType() == Node.ELEMENT_NODE) {
					lessonsText[i] = readLesson(lessonNode);
				}
			}
			return lessonsText;
		} catch (Exception e) {
			e.printStackTrace();
		}
		return null;
	}

	static String[][] readLesson(Node lessonNode) {
		try {
			Element lessonElement = (Element) lessonNode;
			String name;
			Node namedItem = lessonElement.getAttributes().getNamedItem("name");
			name = (namedItem == null) ? "" : namedItem.getNodeValue();
			NodeList listOfMoves = lessonElement.getElementsByTagName("move");
			int totalMoves = listOfMoves.getLength();

			if (DEBUG)
				System.out.println("\t\t\tLesson name: " + name
					+ ", number of moves: " + totalMoves);

			String[][] movesText = new String[totalMoves][];
			for (int i = 0; i < totalMoves; i++) {
				Node moveNode = listOfMoves.item(i);
				if (moveNode.getNodeType() == Node.ELEMENT_NODE) {
					movesText[i] = readMove(moveNode);
				}
			}
			return movesText;
		} catch (Exception e) {
			e.printStackTrace();
		}
		return null;
	}

	static String[] readMove(Node moveNode) {
		try {
			Element moveElement = (Element) moveNode;

			String code = "";
			NodeList codeList = moveElement.getElementsByTagName("code");
			Element codeElement = (Element)codeList.item(0);
			NodeList textCodeList = codeElement.getChildNodes();
			if ((textCodeList.item(0)) != null)
				code = (textCodeList.item(0)).getNodeValue().trim();
			int codeSize = 0;
			if (code != null) {
				String[] codes = code.split(" ");
				codeSize = codes.length;
			}
			/*int[] codePart = new int[codeSize];
			for (int i = 0; i < codeSize; i++) {
				if (codes == null) {
					codePart[i] = -1;
				} else {
					try {
						codePart[i] = Integer.parseInt(codes[i]);
					} catch (Exception e) {
						codePart[i] = -1;
					}
				}
			}*/

			String text = "";
			NodeList textList = moveElement.getElementsByTagName("text");
			Element textElement = (Element)textList.item(0);
			NodeList textTextList = textElement.getChildNodes();
			if ((textTextList.item(0)) != null)
				text = (textTextList.item(0)).getNodeValue().trim();
			String texts[] = null;
			int textSize = 0;
			if (text != null) {
				texts = text.split("\n");
				textSize = texts.length;
			}
			if (texts != null) {
				for (int i = 0; i < textSize; i++) {
					texts[i] = texts[i].trim();
				}
			}

			/*if (DEBUG) {
				System.out.println("\t\t\t\tMove code (" + codeSize + "): "
					+ toString(codePart) );
				System.out.println("\t\t\t\tMove text (" + textSize + "): "
					+ toString(texts));
			}*/
			String[] moveText = new String[codeSize + 1];
			moveText[0] = code;
			if (texts != null) {
				System.arraycopy(texts, 0, moveText, 1, textSize);
			}
			return moveText;
		} catch (Exception e) {
			e.printStackTrace();
		}
		return null;
	}

	/*private static String toString(int[] values) {
		StringBuilder stringBuilder = new StringBuilder();
		for (int i = 0; i < values.length; i++) {
			stringBuilder.append(values[i]);
			if (i != values.length - 1) {
				stringBuilder.append(" ");
			}
		}
		return stringBuilder.toString();
	}

	private static String toString(String[] values) {
		StringBuilder stringBuilder = new StringBuilder();
		for (int i = 0; i < values.length; i++) {
			stringBuilder.append(values[i]);
			if (i != values.length - 1) {
				stringBuilder.append("\n");
			}
		}
		return stringBuilder.toString();
	}*/

	public static String[][][][][][] getLibrary() {
		return libraryText;
	}

	public static String[] getBooks() {
		return libraryBooks;
	}

	public static String[][] getEditions() {
		return libraryEditions;
	}

	public static String[][][] getChapters() {
		return libraryChapters;
	}

	/*public static void main (String args[]) {
		readExamplesXML(doc);
	}*/
}
