package com.sillycycle.bagleyd.abacus.learn;

/*
 * @(#)TeachDialog.java
 *
 * Structure from Java Sourcebook by Ed Anuff, pp 287-289
 *
 * Copyright 2009 - 2014  David A. Bagley, bagleyd AT verizon.net
 *
 * All rights reserved.
 *
 * Permission to use, copy, modify, and distribute this software and
 * its documentation for any purpose and without fee is hereby granted,
 * provided that the above copyright notice appear in all copies and
 * that both that copyright notice and this permission notice appear in
 * supporting documentation, and that the name of the author not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.
 *
 * This program is distributed in the hope that it will be "useful",
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 */

import java.awt.BorderLayout;
import java.awt.Dialog;
import java.awt.Frame;
import java.awt.Panel;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import javax.swing.JLabel;
import javax.swing.JTextField;
import javax.swing.SwingUtilities;

import com.sillycycle.bagleyd.abacus.Abacus;
import com.sillycycle.bagleyd.abacus.AbacusApplet;
import com.sillycycle.bagleyd.abacus.AbacusInterface;
import com.sillycycle.bagleyd.util.ColumnLayout;
import com.sillycycle.bagleyd.util.Icon;

/**
 * The <code>TeachDialog</code> class holds the
 * teach portion of Abacus.
 *
 * @author	David A. Bagley
 * @author	bagleyd AT verizon.net
 * @author	http://www.sillycycle.com/abacus.html
 */

public class TeachDialog extends Dialog {
	private static final long serialVersionUID = 42L;
	static boolean active = false;
	AbacusApplet abacusApplet;
	AbacusTeach abacusTeach;
	Abacus abacus;
	JTextField valueTextField = new JTextField("");
	final int labels = 3;
	JLabel[] teachText = new JLabel[labels];

	public TeachDialog(Frame parent, AbacusApplet applet, Abacus abacus,
			String titleText, String msg,
			int line, Icon icon) {
		super(parent, titleText, true);
		final String big ="                                                                                                                                                                                                             ";

		for (int i = 0; i < labels; i++) {
			if (line == i) {
				teachText[i] = new JLabel(msg + big);
			} else if (teachText[i] == null) {
				teachText[i] = new JLabel(big);
			}
		}
		this.abacusApplet = applet;
		this.abacus = abacus;
		abacusTeach = new AbacusTeach(applet);
		abacusTeach.reset();

		Panel messagePanel = new Panel();
		if (icon != null) {
			messagePanel.add(icon, BorderLayout.LINE_START);
		}
		Panel textPanel = new Panel();
		textPanel.setLayout(new ColumnLayout(0, 0, 0,
			ColumnLayout.LEFT));
		if (valueTextField != null) {
			valueTextField.setColumns(50);
			textPanel.add(valueTextField, BorderLayout.PAGE_START);
		}
		messagePanel.add(textPanel, BorderLayout.CENTER);
		for (int i = 0; i < labels; i++) {
			if (teachText[i] != null) {
				textPanel.add(teachText[i],
					BorderLayout.PAGE_START);
			}
		}
		add(messagePanel, BorderLayout.LINE_START);
		addKeyListener(new KeyListener() {
			@Override
			public void keyPressed(KeyEvent event) {
				abacusApplet.processKey(event.getKeyChar());
			}
			@Override
			public void keyReleased(KeyEvent event) {
				// unused
			}
			@Override
			public void keyTyped(KeyEvent event) {
				// unused
			}
		});
		valueTextField.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent event) {
				teachStep(valueTextField.getText());
			}
		});
		addWindowListener(new WindowListener() {
			@Override
			public void windowClosed(WindowEvent event) {
				// unused
			}
			@Override
			public void windowIconified(WindowEvent event) {
				// unused
			}
			@Override
			public void windowDeiconified(WindowEvent event) {
				// unused
			}
			@Override
			public void windowActivated(WindowEvent event) {
				// unused
			}
			@Override
			public void windowDeactivated(WindowEvent event) {
				// unused
			}
			@Override
			public void windowOpened(WindowEvent event) {
				// unused
			}
			@Override
			public void windowClosing(WindowEvent event) {
				//abacusApplet.processKey('q');
				for (int i = 0; i < labels; i++) {
					teachText[i] = null;
				}
				abacusApplet.killTeachDialog();
			}
		});
		// this solves the problem where the dialog was not getting
		// focus the second time it was displayed
		SwingUtilities.invokeLater(new Runnable() {
			@Override
			public void run() {
				requestFocusInWindow();
			}
		});
		//setSize(getPreferredSize());
		pack();
		//setSize(getMinimumSize());
	}

	void teachStep(String teachBuffer) {
		abacusTeach.teachStep(abacus, teachBuffer,
			AbacusInterface.PRIMARY);
	}

	public void showTeachMessage(String msg, int line) {
		//teachText[line].setSize(getPreferredSize());
		teachText[line].setText(msg);
	}

	public static synchronized boolean getActive() {
		return TeachDialog.active;
	}
}
