package com.sillycycle.bagleyd.util;

/*
 * @(#)ArgumentParser.java
 *
 * From Tip of the Day http://www.devx.com/tips/Tip/13004
 * Simplify Command-Line Argument Parsing
 * May 28, 1999  Whit Stockwell
 */

import java.awt.Color;
import java.util.ArrayList;
import java.util.HashMap;

/**
 * The <code>ArgumentParser</code> class simplifies command-line
 * argument processing by separating arguments into positional
 * parameters and options (any argument starting with "-" or "/" is
 * considered an option, which may specify a value).
 *
 * @author	Whit Stockwell
 */

public class ArgumentParser {
	public ArgumentParser(String[] args) {
	    for (String arg : args) {
		if (arg.startsWith("-") || arg.startsWith("/")) {
		    int loc = arg.indexOf('=');
		    String key = (loc > 0) ? arg.substring(1, loc) : arg.substring(1);
		    String value = (loc > 0) ? arg.substring(loc + 1) : "";
		    this.options.put(key.toLowerCase(), value);
		} else {
		    this.params.add(arg);
		}
	    }
	}

	public boolean hasOption(String opt) {
		return this.options.containsKey(opt.toLowerCase());
	}

	public String getOption(String opt) {
		return this.options.get(opt.toLowerCase());
	}

	public String nextParam() {
		if (this.paramIndex < this.params.size()) {
			return this.params.get(this.paramIndex++);
		}
		return null;
	}

	private final ArrayList<String> params = new ArrayList<>();
	private final HashMap<String, String> options = new HashMap<>();

	private int paramIndex = 0;

	public static int parseOption(String string, int value) {
		if (string == null)
			return value;
		try {
			return Integer.parseInt(string);
		} catch (NumberFormatException e) {
			return value; // ignore garbage
		}
	}

	public static boolean parseOption(String string, boolean value) {
		if (string == null)
			return value;
		try {
			if (string.equalsIgnoreCase("true") ||
					string.equalsIgnoreCase("yes") ||
					string.equalsIgnoreCase("on"))
				return true;
			else if (string.equalsIgnoreCase("false") ||
					string.equalsIgnoreCase("no") ||
					string.equalsIgnoreCase("off"))
				return false;
			return (Integer.parseInt(string) != 0);
		} catch (NumberFormatException e) {
			return value; // ignore garbage
		}
	}

	public static Color parseOption(String string, Color value) {
		if (string == null)
			return value;
		try {
			return Color.decode(string);
		} catch (NumberFormatException e) {
			return value; // ignore garbage
		}
	}
}
